@@ -31,6 +31,7 @@ public:
     static const uint64_t MicroSecPerSec = 1000000UL;
     static const uint64_t NanoSecPerSec = 1000000000UL;
     static const uint64_t NanoSecPerMicroSec = 1000U;
+    static const uint64_t NanoSecPerMilliSec = 1000000UL;
 
     //------------------------------------------------------------------
     // Constructors and Destructors
@@ -97,6 +98,15 @@ public:
         return (m_nano_seconds % NanoSecPerSec) / NanoSecPerMicroSec;
     }
 
+    /// Returns only the fractional portion of the TimeValue rounded down to the
+    /// nearest millisecond (divide by one million).
+    /// @brief Retrieve the milliseconds component;
+    uint32_t
+    milliseconds() const
+    {
+        return m_nano_seconds / NanoSecPerMilliSec;
+    }
+
 protected:
     //------------------------------------------------------------------
     // Classes that inherit from TimeValue can see and modify these
