@@ -24,6 +24,7 @@ namespace lldb_private
     class NativeProcessProtocol;
     class NativeRegisterContext;
     class NativeThreadProtocol;
+    class UnixSignals;
 
     // ---------------------------------------------------------------
     // SP/WP decls.
@@ -33,6 +34,7 @@ namespace lldb_private
     typedef std::weak_ptr<lldb_private::NativeProcessProtocol> NativeProcessProtocolWP;
     typedef std::shared_ptr<lldb_private::NativeRegisterContext> NativeRegisterContextSP;
     typedef std::shared_ptr<lldb_private::NativeThreadProtocol> NativeThreadProtocolSP;
+    typedef std::shared_ptr<lldb_private::UnixSignals> UnixSignalsSP;
 }
 
 #endif // #if defined(__cplusplus)
