@@ -26,6 +26,7 @@
 
 # Python modules:
 import os			# Provide directory and file handling, determine OS information
+import sys			# sys.executable
 import time			# Time access and conversions
 import subprocess 	# Call external programs
 import shutil		# File handling
@@ -113,6 +114,7 @@ def get_header_files( vDictArgs ):
 						"/include/lldb/API/SBSymbolContextList.h",
 						"/include/lldb/API/SBTarget.h",
 						"/include/lldb/API/SBThread.h",
+						"/include/lldb/API/SBThreadCollection.h",
 						"/include/lldb/API/SBType.h",
 						"/include/lldb/API/SBTypeCategory.h",
 						"/include/lldb/API/SBTypeFilter.h",
@@ -187,6 +189,7 @@ def get_interface_files( vDictArgs ):
 						"/scripts/Python/interface/SBSymbolContext.i",
 						"/scripts/Python/interface/SBTarget.i",
 						"/scripts/Python/interface/SBThread.i",
+						"/scripts/Python/interface/SBThreadCollection.i",
 						"/scripts/Python/interface/SBType.i",
 						"/scripts/Python/interface/SBTypeCategory.i",
 						"/scripts/Python/interface/SBTypeFilter.i",
@@ -533,7 +536,7 @@ def run_python_script( vDictArgs, vstrArgs ):
 	strMsg = "";
 	bDbg = vDictArgs.has_key( "-d" );
 	
-	strPy = "python %s" % vstrArgs;
+	strPy = "%s %s" % (sys.executable, vstrArgs);
 	process = subprocess.Popen( strPy, shell=True );
 	strStdOut, strStdErr = process.communicate();
 	nResult = process.returncode;
