@@ -61,13 +61,7 @@ public:
         
     virtual bool
     RegisterIsVolatile (const lldb_private::RegisterInfo *reg_info);
-    
-    virtual bool
-    StackUsesFrames ()
-    {
-        return true;
-    }
-    
+
     // The SysV x86_64 ABI requires that stack frames be 16 byte aligned.
     // When there is a trap handler on the stack, e.g. _sigtramp in userland
     // code, we've seen that the stack pointer is often not aligned properly
@@ -97,12 +91,6 @@ public:
         return true;
     }
 
-    virtual bool
-    FunctionCallsChangeCFA ()
-    {
-        return true;
-    }
-
     virtual const lldb_private::RegisterInfo *
     GetRegisterInfoArray (uint32_t &count);
     //------------------------------------------------------------------
