@@ -42,11 +42,8 @@ using namespace lldb_private;
 bool
 AppleObjCRuntime::GetObjectDescription (Stream &str, ValueObject &valobj)
 {
-    bool is_signed;
-    // ObjC objects can only be pointers, but we extend this to integer types because an expression might just
-    // result in an address, and we should try that to see if the address is an ObjC object.
-    
-    if (!(valobj.IsPointerType() || valobj.IsIntegerType(is_signed)))
+    // ObjC objects can only be pointers
+    if (!valobj.IsPointerType())
         return false;
     
     // Make the argument list: we pass one arg, the address of our pointer, to the print function.
