@@ -42,9 +42,14 @@
 #include "lldb/Target/Thread.h"
 
 #include "AppleObjCRuntimeV2.h"
-#include "AppleObjCTypeVendor.h"
+#include "AppleObjCClassDescriptorV2.h"
+#include "AppleObjCTypeEncodingParser.h"
+#include "AppleObjCDeclVendor.h"
 #include "AppleObjCTrampolineHandler.h"
 
+#include "clang/AST/ASTContext.h"
+#include "clang/AST/DeclObjC.h"
+
 #include <vector>
 
 using namespace lldb;
@@ -342,13 +347,14 @@ AppleObjCRuntimeV2::AppleObjCRuntimeV2 (Process *process,
     m_get_shared_cache_class_info_code(),
     m_get_shared_cache_class_info_args (LLDB_INVALID_ADDRESS),
     m_get_shared_cache_class_info_args_mutex (Mutex::eMutexTypeNormal),
-    m_type_vendor_ap (),
+    m_decl_vendor_ap (),
     m_isa_hash_table_ptr (LLDB_INVALID_ADDRESS),
     m_hash_signature (),
     m_has_object_getClass (false),
     m_loaded_objc_opt (false),
     m_non_pointer_isa_cache_ap(NonPointerISACache::CreateInstance(*this,objc_module_sp)),
-    m_tagged_pointer_vendor_ap(TaggedPointerVendor::CreateInstance(*this,objc_module_sp))
+    m_tagged_pointer_vendor_ap(TaggedPointerVendor::CreateInstance(*this,objc_module_sp)),
+    m_encoding_to_type_sp()
 {
     static const ConstString g_gdb_object_getClass("gdb_object_getClass");
     m_has_object_getClass = (objc_module_sp->FindFirstSymbolWithNameAndType(g_gdb_object_getClass, eSymbolTypeCode) != NULL);
@@ -396,12 +402,12 @@ AppleObjCRuntimeV2::GetDynamicTypeAndAddress (ValueObject &in_value,
                 else
                 {
                     // try to go for a ClangASTType at least
-                    TypeVendor* vendor = GetTypeVendor();
+                    DeclVendor* vendor = GetDeclVendor();
                     if (vendor)
                     {
-                        std::vector<ClangASTType> types;
-                        if (vendor->FindTypes(class_name, false, 1, types) && types.size() && types.at(0).IsValid())
-                            class_type_or_name.SetClangASTType(types.at(0));
+                        std::vector<clang::NamedDecl*> decls;
+                        if (vendor->FindDecls(class_name, false, 1, decls) && decls.size())
+                            class_type_or_name.SetClangASTType(ClangASTContext::GetTypeForDecl(decls[0]));
                     }
                 }
             }
@@ -548,36 +554,52 @@ AppleObjCRuntimeV2::CreateObjectChecker(const char *name)
 size_t
 AppleObjCRuntimeV2::GetByteOffsetForIvar (ClangASTType &parent_ast_type, const char *ivar_name)
 {
+    uint32_t ivar_offset = LLDB_INVALID_IVAR_OFFSET;
+
     const char *class_name = parent_ast_type.GetConstTypeName().AsCString();
+    if (class_name && class_name[0] && ivar_name && ivar_name[0])
+    {
+        //----------------------------------------------------------------------
+        // Make the objective C V2 mangled name for the ivar offset from the
+        // class name and ivar name
+        //----------------------------------------------------------------------
+        std::string buffer("OBJC_IVAR_$_");
+        buffer.append (class_name);
+        buffer.push_back ('.');
+        buffer.append (ivar_name);
+        ConstString ivar_const_str (buffer.c_str());
+        
+        //----------------------------------------------------------------------
+        // Try to get the ivar offset address from the symbol table first using
+        // the name we created above
+        //----------------------------------------------------------------------
+        SymbolContextList sc_list;
+        Target &target = m_process->GetTarget();
+        target.GetImages().FindSymbolsWithNameAndType(ivar_const_str, eSymbolTypeObjCIVar, sc_list);
 
-    if (!class_name || *class_name == '\0' || !ivar_name || *ivar_name == '\0')
-        return LLDB_INVALID_IVAR_OFFSET;
-    
-    std::string buffer("OBJC_IVAR_$_");
-    buffer.append (class_name);
-    buffer.push_back ('.');
-    buffer.append (ivar_name);
-    ConstString ivar_const_str (buffer.c_str());
-    
-    SymbolContextList sc_list;
-    Target &target = m_process->GetTarget();
-    
-    target.GetImages().FindSymbolsWithNameAndType(ivar_const_str, eSymbolTypeObjCIVar, sc_list);
+        addr_t ivar_offset_address = LLDB_INVALID_ADDRESS;
 
-    SymbolContext ivar_offset_symbol;
-    if (sc_list.GetSize() != 1 
-        || !sc_list.GetContextAtIndex(0, ivar_offset_symbol) 
-        || ivar_offset_symbol.symbol == NULL)
-        return LLDB_INVALID_IVAR_OFFSET;
-    
-    addr_t ivar_offset_address = ivar_offset_symbol.symbol->GetAddress().GetLoadAddress (&target);
-    
-    Error error;
-    
-    uint32_t ivar_offset = m_process->ReadUnsignedIntegerFromMemory (ivar_offset_address, 
-                                                                     4, 
-                                                                     LLDB_INVALID_IVAR_OFFSET, 
-                                                                     error);
+        Error error;
+        SymbolContext ivar_offset_symbol;
+        if (sc_list.GetSize() == 1 && sc_list.GetContextAtIndex(0, ivar_offset_symbol))
+        {
+            if (ivar_offset_symbol.symbol)
+                ivar_offset_address = ivar_offset_symbol.symbol->GetAddress().GetLoadAddress (&target);
+        }
+
+        //----------------------------------------------------------------------
+        // If we didn't get the ivar offset address from the symbol table, fall
+        // back to getting it from the runtime
+        //----------------------------------------------------------------------
+        if (ivar_offset_address == LLDB_INVALID_ADDRESS)
+            ivar_offset_address = LookupRuntimeSymbol(ivar_const_str);
+
+        if (ivar_offset_address != LLDB_INVALID_ADDRESS)
+            ivar_offset = m_process->ReadUnsignedIntegerFromMemory (ivar_offset_address,
+                                                                    4,
+                                                                    LLDB_INVALID_IVAR_OFFSET,
+                                                                    error);
+    }
     return ivar_offset;
 }
 
@@ -855,757 +877,6 @@ AppleObjCRuntimeV2::HashTableSignature::NeedsUpdate (Process *process, AppleObjC
     return true;
 }
 
-class ClassDescriptorV2 : public ObjCLanguageRuntime::ClassDescriptor
-{
-public:
-    friend class lldb_private::AppleObjCRuntimeV2;
-    
-private:
-    // The constructor should only be invoked by the runtime as it builds its caches
-    // or populates them.  A ClassDescriptorV2 should only ever exist in a cache.
-    ClassDescriptorV2 (AppleObjCRuntimeV2 &runtime, ObjCLanguageRuntime::ObjCISA isa, const char *name) :
-        m_runtime (runtime),
-        m_objc_class_ptr (isa),
-        m_name (name)
-    {
-    }
-
-public:
-    virtual ConstString
-    GetClassName ()
-    {
-        if (!m_name)
-        {
-            lldb_private::Process *process = m_runtime.GetProcess();
-
-            if (process)
-            {
-                std::unique_ptr<objc_class_t> objc_class;
-                std::unique_ptr<class_ro_t> class_ro;
-                std::unique_ptr<class_rw_t> class_rw;
-                
-                if (!Read_objc_class(process, objc_class))
-                    return m_name;
-                if (!Read_class_row(process, *objc_class, class_ro, class_rw))
-                    return m_name;
-                
-                m_name = ConstString(class_ro->m_name.c_str());
-            }
-        }
-        return m_name;
-    }
-    
-    virtual ObjCLanguageRuntime::ClassDescriptorSP
-    GetSuperclass ()
-    {
-        lldb_private::Process *process = m_runtime.GetProcess();
-
-        if (!process)
-            return ObjCLanguageRuntime::ClassDescriptorSP();
-        
-        std::unique_ptr<objc_class_t> objc_class;
-
-        if (!Read_objc_class(process, objc_class))
-            return ObjCLanguageRuntime::ClassDescriptorSP();
-
-        return m_runtime.ObjCLanguageRuntime::GetClassDescriptorFromISA(objc_class->m_superclass);
-    }
-    
-    virtual bool
-    IsValid ()
-    {
-        return true;    // any Objective-C v2 runtime class descriptor we vend is valid
-    }
-    
-    // a custom descriptor is used for tagged pointers
-    virtual bool
-    GetTaggedPointerInfo (uint64_t* info_bits = NULL,
-                          uint64_t* value_bits = NULL,
-                          uint64_t* payload = NULL)
-    {
-        return false;
-    }
-    
-    virtual uint64_t
-    GetInstanceSize ()
-    {
-        lldb_private::Process *process = m_runtime.GetProcess();
-        
-        if (process)
-        {
-            std::unique_ptr<objc_class_t> objc_class;
-            std::unique_ptr<class_ro_t> class_ro;
-            std::unique_ptr<class_rw_t> class_rw;
-            
-            if (!Read_objc_class(process, objc_class))
-                return 0;
-            if (!Read_class_row(process, *objc_class, class_ro, class_rw))
-                return 0;
-            
-            return class_ro->m_instanceSize;
-        }
-        
-        return 0;
-    }
-    
-    virtual ObjCLanguageRuntime::ObjCISA
-    GetISA ()
-    {        
-        return m_objc_class_ptr;
-    }
-    
-    virtual bool
-    Describe (std::function <void (ObjCLanguageRuntime::ObjCISA)> const &superclass_func,
-              std::function <bool (const char *, const char *)> const &instance_method_func,
-              std::function <bool (const char *, const char *)> const &class_method_func,
-              std::function <bool (const char *, const char *, lldb::addr_t, uint64_t)> const &ivar_func)
-    {
-        lldb_private::Process *process = m_runtime.GetProcess();
-
-        std::unique_ptr<objc_class_t> objc_class;
-        std::unique_ptr<class_ro_t> class_ro;
-        std::unique_ptr<class_rw_t> class_rw;
-        
-        if (!Read_objc_class(process, objc_class))
-            return 0;
-        if (!Read_class_row(process, *objc_class, class_ro, class_rw))
-            return 0;
-    
-        static ConstString NSObject_name("NSObject");
-        
-        if (m_name != NSObject_name && superclass_func)
-            superclass_func(objc_class->m_superclass);
-        
-        if (instance_method_func)
-        {
-            std::unique_ptr<method_list_t> base_method_list;
-            
-            base_method_list.reset(new method_list_t);
-            if (!base_method_list->Read(process, class_ro->m_baseMethods_ptr))
-                return false;
-            
-            if (base_method_list->m_entsize != method_t::GetSize(process))
-                return false;
-            
-            std::unique_ptr<method_t> method;
-            method.reset(new method_t);
-            
-            for (uint32_t i = 0, e = base_method_list->m_count; i < e; ++i)
-            {
-                method->Read(process, base_method_list->m_first_ptr + (i * base_method_list->m_entsize));
-                
-                if (instance_method_func(method->m_name.c_str(), method->m_types.c_str()))
-                    break;
-            }
-        }
-        
-        if (class_method_func)
-        {
-            ClassDescriptorV2 metaclass(m_runtime, objc_class->m_isa, NULL); // The metaclass is not in the cache
-            
-            // We don't care about the metaclass's superclass, or its class methods.  Its instance methods are
-            // our class methods.
-            
-            metaclass.Describe(std::function <void (ObjCLanguageRuntime::ObjCISA)> (nullptr),
-                               class_method_func,
-                               std::function <bool (const char *, const char *)> (nullptr),
-                               std::function <bool (const char *, const char *, lldb::addr_t, uint64_t)> (nullptr));
-        }
-        
-        if (ivar_func)
-        {
-            ivar_list_t ivar_list;
-            if (!ivar_list.Read(process, class_ro->m_ivars_ptr))
-                return false;
-            
-            if (ivar_list.m_entsize != ivar_t::GetSize(process))
-                return false;
-            
-            ivar_t ivar;
-            
-            for (uint32_t i = 0, e = ivar_list.m_count; i < e; ++i)
-            {
-                ivar.Read(process, ivar_list.m_first_ptr + (i * ivar_list.m_entsize));
-                
-                if (ivar_func(ivar.m_name.c_str(), ivar.m_type.c_str(), ivar.m_offset_ptr, ivar.m_size))
-                    break;
-            }
-        }
-            
-        return true;
-    }
-    
-    virtual
-    ~ClassDescriptorV2 ()
-    {
-    }
-        
-private:
-    static const uint32_t RW_REALIZED = (1 << 31);
-    
-    struct objc_class_t {
-        ObjCLanguageRuntime::ObjCISA    m_isa;              // The class's metaclass.
-        ObjCLanguageRuntime::ObjCISA    m_superclass;
-        lldb::addr_t                    m_cache_ptr;
-        lldb::addr_t                    m_vtable_ptr;
-        lldb::addr_t                    m_data_ptr;
-        uint8_t                         m_flags;
-        
-        objc_class_t () :
-            m_isa (0),
-            m_superclass (0),
-            m_cache_ptr (0),
-            m_vtable_ptr (0),
-            m_data_ptr (0),
-            m_flags (0)
-        {
-        }
-        
-        void
-        Clear()
-        {
-            m_isa = 0;
-            m_superclass = 0;
-            m_cache_ptr = 0;
-            m_vtable_ptr = 0;
-            m_data_ptr = 0;
-            m_flags = 0;
-        }
-
-        bool Read(Process *process, lldb::addr_t addr)
-        {
-            size_t ptr_size = process->GetAddressByteSize();
-            
-            size_t objc_class_size = ptr_size   // uintptr_t isa;
-            + ptr_size   // Class superclass;
-            + ptr_size   // void *cache;
-            + ptr_size   // IMP *vtable;
-            + ptr_size;  // uintptr_t data_NEVER_USE;
-            
-            DataBufferHeap objc_class_buf (objc_class_size, '\0');
-            Error error;
-            
-            process->ReadMemory(addr, objc_class_buf.GetBytes(), objc_class_size, error);
-            if (error.Fail())
-            {
-                return false;
-            }
-            
-            DataExtractor extractor(objc_class_buf.GetBytes(), objc_class_size, process->GetByteOrder(), process->GetAddressByteSize());
-            
-            lldb::offset_t cursor = 0;
-            
-            m_isa           = extractor.GetAddress_unchecked(&cursor);   // uintptr_t isa;
-            m_superclass    = extractor.GetAddress_unchecked(&cursor);   // Class superclass;
-            m_cache_ptr     = extractor.GetAddress_unchecked(&cursor);   // void *cache;
-            m_vtable_ptr    = extractor.GetAddress_unchecked(&cursor);   // IMP *vtable;
-            lldb::addr_t data_NEVER_USE = extractor.GetAddress_unchecked(&cursor);   // uintptr_t data_NEVER_USE;
-            
-            m_flags         = (uint8_t)(data_NEVER_USE & (lldb::addr_t)3);
-            m_data_ptr      = data_NEVER_USE & ~(lldb::addr_t)3;
-            
-            return true;
-        }
-    };
-    
-    struct class_ro_t {
-        uint32_t                        m_flags;
-        uint32_t                        m_instanceStart;
-        uint32_t                        m_instanceSize;
-        uint32_t                        m_reserved;
-        
-        lldb::addr_t                    m_ivarLayout_ptr;
-        lldb::addr_t                    m_name_ptr;
-        lldb::addr_t                    m_baseMethods_ptr;
-        lldb::addr_t                    m_baseProtocols_ptr;
-        lldb::addr_t                    m_ivars_ptr;
-        
-        lldb::addr_t                    m_weakIvarLayout_ptr;
-        lldb::addr_t                    m_baseProperties_ptr;
-        
-        std::string                     m_name;
-        
-        bool Read(Process *process, lldb::addr_t addr)
-        {
-            size_t ptr_size = process->GetAddressByteSize();
-            
-            size_t size = sizeof(uint32_t)             // uint32_t flags;
-            + sizeof(uint32_t)                         // uint32_t instanceStart;
-            + sizeof(uint32_t)                         // uint32_t instanceSize;
-            + (ptr_size == 8 ? sizeof(uint32_t) : 0)   // uint32_t reserved; // __LP64__ only
-            + ptr_size                                 // const uint8_t *ivarLayout;
-            + ptr_size                                 // const char *name;
-            + ptr_size                                 // const method_list_t *baseMethods;
-            + ptr_size                                 // const protocol_list_t *baseProtocols;
-            + ptr_size                                 // const ivar_list_t *ivars;
-            + ptr_size                                 // const uint8_t *weakIvarLayout;
-            + ptr_size;                                // const property_list_t *baseProperties;
-            
-            DataBufferHeap buffer (size, '\0');
-            Error error;
-            
-            process->ReadMemory(addr, buffer.GetBytes(), size, error);
-            if (error.Fail())
-            {
-                return false;
-            }
-            
-            DataExtractor extractor(buffer.GetBytes(), size, process->GetByteOrder(), process->GetAddressByteSize());
-            
-            lldb::offset_t cursor = 0;
-            
-            m_flags             = extractor.GetU32_unchecked(&cursor);
-            m_instanceStart     = extractor.GetU32_unchecked(&cursor);
-            m_instanceSize      = extractor.GetU32_unchecked(&cursor);
-            if (ptr_size == 8)
-                m_reserved      = extractor.GetU32_unchecked(&cursor);
-            else
-                m_reserved      = 0;
-            m_ivarLayout_ptr     = extractor.GetAddress_unchecked(&cursor);
-            m_name_ptr           = extractor.GetAddress_unchecked(&cursor);
-            m_baseMethods_ptr    = extractor.GetAddress_unchecked(&cursor);
-            m_baseProtocols_ptr  = extractor.GetAddress_unchecked(&cursor);
-            m_ivars_ptr          = extractor.GetAddress_unchecked(&cursor);
-            m_weakIvarLayout_ptr = extractor.GetAddress_unchecked(&cursor);
-            m_baseProperties_ptr = extractor.GetAddress_unchecked(&cursor);
-            
-            DataBufferHeap name_buf(1024, '\0');
-            
-            process->ReadCStringFromMemory(m_name_ptr, (char*)name_buf.GetBytes(), name_buf.GetByteSize(), error);
-            
-            if (error.Fail())
-            {
-                return false;
-            }
-            
-            m_name.assign((char*)name_buf.GetBytes());
-                
-            return true;
-        }
-    };
-    
-    struct class_rw_t {
-        uint32_t                        m_flags;
-        uint32_t                        m_version;
-        
-        lldb::addr_t                    m_ro_ptr;
-        union {
-            lldb::addr_t                m_method_list_ptr;
-            lldb::addr_t                m_method_lists_ptr;
-        };
-        lldb::addr_t                    m_properties_ptr;
-        lldb::addr_t                    m_protocols_ptr;
-        
-        ObjCLanguageRuntime::ObjCISA    m_firstSubclass;
-        ObjCLanguageRuntime::ObjCISA    m_nextSiblingClass;
-        
-        bool Read(Process *process, lldb::addr_t addr)
-        {
-            size_t ptr_size = process->GetAddressByteSize();
-            
-            size_t size = sizeof(uint32_t)  // uint32_t flags;
-            + sizeof(uint32_t)  // uint32_t version;
-            + ptr_size          // const class_ro_t *ro;
-            + ptr_size          // union { method_list_t **method_lists; method_list_t *method_list; };
-            + ptr_size          // struct chained_property_list *properties;
-            + ptr_size          // const protocol_list_t **protocols;
-            + ptr_size          // Class firstSubclass;
-            + ptr_size;         // Class nextSiblingClass;
-            
-            DataBufferHeap buffer (size, '\0');
-            Error error;
-            
-            process->ReadMemory(addr, buffer.GetBytes(), size, error);
-            if (error.Fail())
-            {
-                return false;
-            }
-            
-            DataExtractor extractor(buffer.GetBytes(), size, process->GetByteOrder(), process->GetAddressByteSize());
-            
-            lldb::offset_t cursor = 0;
-            
-            m_flags             = extractor.GetU32_unchecked(&cursor);
-            m_version           = extractor.GetU32_unchecked(&cursor);
-            m_ro_ptr            = extractor.GetAddress_unchecked(&cursor);
-            m_method_list_ptr   = extractor.GetAddress_unchecked(&cursor);
-            m_properties_ptr    = extractor.GetAddress_unchecked(&cursor);
-            m_firstSubclass     = extractor.GetAddress_unchecked(&cursor);
-            m_nextSiblingClass  = extractor.GetAddress_unchecked(&cursor);
-            
-            return true;
-        }
-    };
-    
-    struct method_list_t
-    {
-        uint32_t        m_entsize;
-        uint32_t        m_count;
-        lldb::addr_t    m_first_ptr;
-        
-        bool Read(Process *process, lldb::addr_t addr)
-        {
-            size_t size = sizeof(uint32_t)  // uint32_t entsize_NEVER_USE;
-            + sizeof(uint32_t); // uint32_t count;
-            
-            DataBufferHeap buffer (size, '\0');
-            Error error;
-            
-            process->ReadMemory(addr, buffer.GetBytes(), size, error);
-            if (error.Fail())
-            {
-                return false;
-            }
-            
-            DataExtractor extractor(buffer.GetBytes(), size, process->GetByteOrder(), process->GetAddressByteSize());
-            
-            lldb::offset_t cursor = 0;
-            
-            m_entsize   = extractor.GetU32_unchecked(&cursor) & ~(uint32_t)3;
-            m_count     = extractor.GetU32_unchecked(&cursor);
-            m_first_ptr  = addr + cursor;
-            
-            return true;
-        }
-    };
-    
-    struct method_t
-    {
-        lldb::addr_t    m_name_ptr;
-        lldb::addr_t    m_types_ptr;
-        lldb::addr_t    m_imp_ptr;
-        
-        std::string     m_name;
-        std::string     m_types;
-        
-        static size_t GetSize(Process *process)
-        {
-            size_t ptr_size = process->GetAddressByteSize();
-            
-            return ptr_size     // SEL name;
-            + ptr_size   // const char *types;
-            + ptr_size;  // IMP imp;
-        }
-        
-        bool Read(Process *process, lldb::addr_t addr)
-        {
-            size_t size = GetSize(process);
-            
-            DataBufferHeap buffer (size, '\0');
-            Error error;
-            
-            process->ReadMemory(addr, buffer.GetBytes(), size, error);
-            if (error.Fail())
-            {
-                return false;
-            }
-            
-            DataExtractor extractor(buffer.GetBytes(), size, process->GetByteOrder(), process->GetAddressByteSize());
-            
-            lldb::offset_t cursor = 0;
-            
-            m_name_ptr   = extractor.GetAddress_unchecked(&cursor);
-            m_types_ptr  = extractor.GetAddress_unchecked(&cursor);
-            m_imp_ptr    = extractor.GetAddress_unchecked(&cursor);
-            
-            const size_t buffer_size = 1024;
-            size_t count;
-            
-            DataBufferHeap string_buf(buffer_size, 0);
-            
-            count = process->ReadCStringFromMemory(m_name_ptr, (char*)string_buf.GetBytes(), buffer_size, error);
-            m_name.assign((char*)string_buf.GetBytes(), count);
-            
-            count = process->ReadCStringFromMemory(m_types_ptr, (char*)string_buf.GetBytes(), buffer_size, error);
-            m_types.assign((char*)string_buf.GetBytes(), count);
-            
-            return true;
-        }
-    };
-    
-    struct ivar_list_t
-    {
-        uint32_t        m_entsize;
-        uint32_t        m_count;
-        lldb::addr_t    m_first_ptr;
-        
-        bool Read(Process *process, lldb::addr_t addr)
-        {
-            size_t size = sizeof(uint32_t)  // uint32_t entsize;
-                        + sizeof(uint32_t); // uint32_t count;
-            
-            DataBufferHeap buffer (size, '\0');
-            Error error;
-            
-            process->ReadMemory(addr, buffer.GetBytes(), size, error);
-            if (error.Fail())
-            {
-                return false;
-            }
-            
-            DataExtractor extractor(buffer.GetBytes(), size, process->GetByteOrder(), process->GetAddressByteSize());
-            
-            lldb::offset_t cursor = 0;
-            
-            m_entsize   = extractor.GetU32_unchecked(&cursor);
-            m_count     = extractor.GetU32_unchecked(&cursor);
-            m_first_ptr = addr + cursor;
-            
-            return true;
-        }
-    };
-    
-    struct ivar_t
-    {
-        lldb::addr_t    m_offset_ptr;
-        lldb::addr_t    m_name_ptr;
-        lldb::addr_t    m_type_ptr;
-        uint32_t        m_alignment;
-        uint32_t        m_size;
-        
-        std::string     m_name;
-        std::string     m_type;
-        
-        static size_t GetSize(Process *process)
-        {
-            size_t ptr_size = process->GetAddressByteSize();
-            
-            return ptr_size             // uintptr_t *offset;
-                 + ptr_size             // const char *name;
-                 + ptr_size             // const char *type;
-                 + sizeof(uint32_t)     // uint32_t alignment;
-                 + sizeof(uint32_t);    // uint32_t size;
-        }
-        
-        bool Read(Process *process, lldb::addr_t addr)
-        {
-            size_t size = GetSize(process);
-            
-            DataBufferHeap buffer (size, '\0');
-            Error error;
-            
-            process->ReadMemory(addr, buffer.GetBytes(), size, error);
-            if (error.Fail())
-            {
-                return false;
-            }
-            
-            DataExtractor extractor(buffer.GetBytes(), size, process->GetByteOrder(), process->GetAddressByteSize());
-            
-            lldb::offset_t cursor = 0;
-            
-            m_offset_ptr = extractor.GetAddress_unchecked(&cursor);
-            m_name_ptr   = extractor.GetAddress_unchecked(&cursor);
-            m_type_ptr   = extractor.GetAddress_unchecked(&cursor);
-            m_alignment  = extractor.GetU32_unchecked(&cursor);
-            m_size       = extractor.GetU32_unchecked(&cursor);
-            
-            const size_t buffer_size = 1024;
-            size_t count;
-            
-            DataBufferHeap string_buf(buffer_size, 0);
-            
-            count = process->ReadCStringFromMemory(m_name_ptr, (char*)string_buf.GetBytes(), buffer_size, error);
-            m_name.assign((char*)string_buf.GetBytes(), count);
-            
-            count = process->ReadCStringFromMemory(m_type_ptr, (char*)string_buf.GetBytes(), buffer_size, error);
-            m_type.assign((char*)string_buf.GetBytes(), count);
-            
-            return true;
-        }
-    };
-    
-    bool Read_objc_class (Process* process, std::unique_ptr<objc_class_t> &objc_class)
-    {
-        objc_class.reset(new objc_class_t);
-        
-        bool ret = objc_class->Read (process, m_objc_class_ptr);
-        
-        if (!ret)
-            objc_class.reset();
-        
-        return ret;
-    }
-    
-    bool Read_class_row (Process* process, const objc_class_t &objc_class, std::unique_ptr<class_ro_t> &class_ro, std::unique_ptr<class_rw_t> &class_rw)
-    {
-        class_ro.reset();
-        class_rw.reset();
-        
-        Error error;
-        uint32_t class_row_t_flags = process->ReadUnsignedIntegerFromMemory(objc_class.m_data_ptr, sizeof(uint32_t), 0, error);
-        if (!error.Success())
-            return false;
-
-        if (class_row_t_flags & RW_REALIZED)
-        {
-            class_rw.reset(new class_rw_t);
-            
-            if (!class_rw->Read(process, objc_class.m_data_ptr))
-            {
-                class_rw.reset();
-                return false;
-            }
-            
-            class_ro.reset(new class_ro_t);
-            
-            if (!class_ro->Read(process, class_rw->m_ro_ptr))
-            {
-                class_rw.reset();
-                class_ro.reset();
-                return false;
-            }
-        }
-        else
-        {
-            class_ro.reset(new class_ro_t);
-            
-            if (!class_ro->Read(process, objc_class.m_data_ptr))
-            {
-                class_ro.reset();
-                return false;
-            }
-        }
-        
-        return true;
-    }
-
-    AppleObjCRuntimeV2 &m_runtime;          // The runtime, so we can read information lazily.
-    lldb::addr_t        m_objc_class_ptr;   // The address of the objc_class_t.  (I.e., objects of this class type have this as their ISA)
-    ConstString         m_name;             // May be NULL
-};
-
-// tagged pointer descriptor
-class ClassDescriptorV2Tagged : public ObjCLanguageRuntime::ClassDescriptor
-{
-public:
-    ClassDescriptorV2Tagged (ConstString class_name,
-                             uint64_t payload)
-    {
-        m_name = class_name;
-        if (!m_name)
-        {
-            m_valid = false;
-            return;
-        }
-        m_valid = true;
-        m_payload = payload;
-        m_info_bits = (m_payload & 0xF0ULL) >> 4;
-        m_value_bits = (m_payload & ~0x0000000000000000FFULL) >> 8;
-    }
-    
-    ClassDescriptorV2Tagged (ObjCLanguageRuntime::ClassDescriptorSP actual_class_sp,
-                             uint64_t payload)
-    {
-        if (!actual_class_sp)
-        {
-            m_valid = false;
-            return;
-        }
-        m_name = actual_class_sp->GetClassName();
-        if (!m_name)
-        {
-            m_valid = false;
-            return;
-        }
-        m_valid = true;
-        m_payload = payload;
-        m_info_bits = (m_payload & 0x0FULL);
-        m_value_bits = (m_payload & ~0x0FULL) >> 4;
-    }
-    
-    virtual ConstString
-    GetClassName ()
-    {
-        return m_name;
-    }
-    
-    virtual ObjCLanguageRuntime::ClassDescriptorSP
-    GetSuperclass ()
-    {
-        // tagged pointers can represent a class that has a superclass, but since that information is not
-        // stored in the object itself, we would have to query the runtime to discover the hierarchy
-        // for the time being, we skip this step in the interest of static discovery
-        return ObjCLanguageRuntime::ClassDescriptorSP();
-    }
-    
-    virtual bool
-    IsValid ()
-    {
-        return m_valid;
-    }
-    
-    virtual bool
-    IsKVO ()
-    {
-        return false; // tagged pointers are not KVO'ed
-    }
-    
-    virtual bool
-    IsCFType ()
-    {
-        return false; // tagged pointers are not CF objects
-    }
-    
-    virtual bool
-    GetTaggedPointerInfo (uint64_t* info_bits = NULL,
-                          uint64_t* value_bits = NULL,
-                          uint64_t* payload = NULL)
-    {
-        if (info_bits)
-            *info_bits = GetInfoBits();
-        if (value_bits)
-            *value_bits = GetValueBits();
-        if (payload)
-            *payload = GetPayload();
-        return true;
-    }
-    
-    virtual uint64_t
-    GetInstanceSize ()
-    {
-        return (IsValid() ? m_pointer_size : 0);
-    }
-    
-    virtual ObjCLanguageRuntime::ObjCISA
-    GetISA ()
-    {
-        return 0; // tagged pointers have no ISA
-    }
-    
-    // these calls are not part of any formal tagged pointers specification
-    virtual uint64_t
-    GetValueBits ()
-    {
-        return (IsValid() ? m_value_bits : 0);
-    }
-    
-    virtual uint64_t
-    GetInfoBits ()
-    {
-        return (IsValid() ? m_info_bits : 0);
-    }
-    
-    virtual uint64_t
-    GetPayload ()
-    {
-        return (IsValid() ? m_payload : 0);
-    }
-    
-    virtual
-    ~ClassDescriptorV2Tagged ()
-    {}
-
-private:
-    ConstString m_name;
-    uint8_t m_pointer_size;
-    bool m_valid;
-    uint64_t m_info_bits;
-    uint64_t m_value_bits;
-    uint64_t m_payload;
-
-};
-
 ObjCLanguageRuntime::ClassDescriptorSP
 AppleObjCRuntimeV2::GetClassDescriptorFromISA (ObjCISA isa)
 {
@@ -1621,6 +892,18 @@ ObjCLanguageRuntime::ClassDescriptorSP
 AppleObjCRuntimeV2::GetClassDescriptor (ValueObject& valobj)
 {
     ClassDescriptorSP objc_class_sp;
+    if (valobj.IsBaseClass())
+    {
+        ValueObject *parent = valobj.GetParent();
+        // if I am my own parent, bail out of here fast..
+        if (parent && parent != &valobj)
+        {
+            ClassDescriptorSP parent_descriptor_sp = GetClassDescriptor(*parent);
+            if (parent_descriptor_sp)
+                return parent_descriptor_sp->GetSuperclass();
+        }
+        return nullptr;
+    }
     // if we get an invalid VO (which might still happen when playing around
     // with pointers returned by the expression parser, don't consider this
     // a valid ObjC object)
@@ -2310,13 +1593,13 @@ AppleObjCRuntimeV2::GetActualTypeName(ObjCLanguageRuntime::ObjCISA isa)
     return ObjCLanguageRuntime::GetActualTypeName(isa);
 }
 
-TypeVendor *
-AppleObjCRuntimeV2::GetTypeVendor()
+DeclVendor *
+AppleObjCRuntimeV2::GetDeclVendor()
 {
-    if (!m_type_vendor_ap.get())
-        m_type_vendor_ap.reset(new AppleObjCTypeVendor(*this));
+    if (!m_decl_vendor_ap.get())
+        m_decl_vendor_ap.reset(new AppleObjCDeclVendor(*this));
     
-    return m_type_vendor_ap.get();
+    return m_decl_vendor_ap.get();
 }
 
 lldb::addr_t
@@ -2651,3 +1934,22 @@ AppleObjCRuntimeV2::NonPointerISACache::EvaluateNonPointerISA (ObjCISA isa, ObjC
     }
     return false;
 }
+
+ObjCLanguageRuntime::EncodingToTypeSP
+AppleObjCRuntimeV2::GetEncodingToType ()
+{
+    if (!m_encoding_to_type_sp)
+        m_encoding_to_type_sp.reset(new AppleObjCTypeEncodingParser(*this));
+    return m_encoding_to_type_sp;
+}
+
+lldb_private::AppleObjCRuntime::ObjCISA
+AppleObjCRuntimeV2::GetPointerISA (ObjCISA isa)
+{
+    ObjCISA ret = isa;
+    
+    if (m_non_pointer_isa_cache_ap)
+        m_non_pointer_isa_cache_ap->EvaluateNonPointerISA(isa, ret);
+    
+    return ret;
+}
