@@ -133,6 +133,9 @@ public:
     virtual uint32_t
     GetAddressByteSize() const;
 
+    virtual lldb::AddressClass
+    GetAddressClass (lldb::addr_t file_addr);
+
     virtual lldb_private::Symtab *
     GetSymtab();
 
