@@ -14,6 +14,7 @@
 // C++ Includes
 // Other libraries and framework includes
 // Project includes
+#include "lldb/Host/FileSpec.h"
 #include "Plugins/Platform/POSIX/PlatformPOSIX.h"
 
 class PlatformDarwin : public PlatformPOSIX
@@ -27,96 +28,46 @@ public:
     //------------------------------------------------------------
     // lldb_private::Platform functions
     //------------------------------------------------------------
-    virtual lldb_private::Error
-    ResolveExecutable (const lldb_private::FileSpec &exe_file,
-                       const lldb_private::ArchSpec &arch,
+    lldb_private::Error
+    ResolveExecutable (const lldb_private::ModuleSpec &module_spec,
                        lldb::ModuleSP &module_sp,
-                       const lldb_private::FileSpecList *module_search_paths_ptr);
+                       const lldb_private::FileSpecList *module_search_paths_ptr) override;
 
-    virtual lldb_private::Error
+    lldb_private::Error
     ResolveSymbolFile (lldb_private::Target &target,
                        const lldb_private::ModuleSpec &sym_spec,
-                       lldb_private::FileSpec &sym_file);
+                       lldb_private::FileSpec &sym_file) override;
 
     lldb_private::FileSpecList
     LocateExecutableScriptingResources (lldb_private::Target *target,
-                                        lldb_private::Module &module);
+                                        lldb_private::Module &module,
+                                        lldb_private::Stream* feedback_stream) override;
     
-    virtual lldb_private::Error
+    lldb_private::Error
     GetSharedModule (const lldb_private::ModuleSpec &module_spec,
                      lldb::ModuleSP &module_sp,
                      const lldb_private::FileSpecList *module_search_paths_ptr,
                      lldb::ModuleSP *old_module_sp_ptr,
-                     bool *did_create_ptr);
+                     bool *did_create_ptr) override;
 
-    virtual size_t
+    size_t
     GetSoftwareBreakpointTrapOpcode (lldb_private::Target &target, 
-                                     lldb_private::BreakpointSite *bp_site);
+                                     lldb_private::BreakpointSite *bp_site) override;
 
-    virtual bool
-    GetRemoteOSVersion ();
-
-    virtual bool
-    GetRemoteOSBuildString (std::string &s);
-    
-    virtual bool
-    GetRemoteOSKernelDescription (std::string &s);
-
-    // Remote Platform subclasses need to override this function
-    virtual lldb_private::ArchSpec
-    GetRemoteSystemArchitecture ();
-
-    virtual bool
-    IsConnected () const;
-
-    virtual lldb_private::Error
-    ConnectRemote (lldb_private::Args& args);
-
-    virtual lldb_private::Error
-    DisconnectRemote ();
-
-    virtual const char *
-    GetHostname ();
-
-    virtual const char *
-    GetUserName (uint32_t uid);
-    
-    virtual const char *
-    GetGroupName (uint32_t gid);
-
-    virtual bool
+    bool
     GetProcessInfo (lldb::pid_t pid, 
-                    lldb_private::ProcessInstanceInfo &proc_info);
+                    lldb_private::ProcessInstanceInfo &proc_info) override;
     
-    virtual lldb::BreakpointSP
-    SetThreadCreationBreakpoint (lldb_private::Target &target);
+    lldb::BreakpointSP
+    SetThreadCreationBreakpoint (lldb_private::Target &target) override;
 
-    virtual uint32_t
+    uint32_t
     FindProcesses (const lldb_private::ProcessInstanceInfoMatch &match_info,
-                   lldb_private::ProcessInstanceInfoList &process_infos);
-    
-    virtual lldb_private::Error
-    LaunchProcess (lldb_private::ProcessLaunchInfo &launch_info);
+                   lldb_private::ProcessInstanceInfoList &process_infos) override;
 
-    virtual lldb::ProcessSP
-    DebugProcess (lldb_private::ProcessLaunchInfo &launch_info,
-                  lldb_private::Debugger &debugger,
-                  lldb_private::Target *target,       // Can be NULL, if NULL create a new target, else use existing one
-                  lldb_private::Listener &listener,
-                  lldb_private::Error &error);
-
-    virtual lldb::ProcessSP
-    Attach (lldb_private::ProcessAttachInfo &attach_info,
-            lldb_private::Debugger &debugger,
-            lldb_private::Target *target,       // Can be NULL, if NULL create a new target, else use existing one
-            lldb_private::Listener &listener, 
-            lldb_private::Error &error);
-
-    virtual bool
-    ModuleIsExcludedForNonModuleSpecificSearches (lldb_private::Target &target, const lldb::ModuleSP &module_sp);
-    
-    virtual size_t
-    GetEnvironment (lldb_private::StringList &environment);
+    bool
+    ModuleIsExcludedForUnconstrainedSearches(lldb_private::Target &target,
+						 const lldb::ModuleSP &module_sp) override;
 
     bool
     ARMGetSupportedArchitectureAtIndex (uint32_t idx, lldb_private::ArchSpec &arch);
@@ -124,11 +75,14 @@ public:
     bool 
     x86GetSupportedArchitectureAtIndex (uint32_t idx, lldb_private::ArchSpec &arch);
     
-    virtual int32_t
-    GetResumeCountForLaunchInfo (lldb_private::ProcessLaunchInfo &launch_info);
+    int32_t
+    GetResumeCountForLaunchInfo (lldb_private::ProcessLaunchInfo &launch_info) override;
 
-    virtual void
-    CalculateTrapHandlerSymbolNames ();
+    void
+    CalculateTrapHandlerSymbolNames () override;
+    
+    bool
+    SupportsModules () override { return true; }
 
 protected:
 
@@ -144,6 +98,39 @@ protected:
                                    const lldb_private::FileSpecList *module_search_paths_ptr,
                                    lldb::ModuleSP *old_module_sp_ptr,
                                    bool *did_create_ptr);
+    
+    
+    enum class SDKType {
+        MacOSX = 0,
+        iPhoneSimulator,
+        iPhoneOS,
+    };
+    
+    static bool
+    SDKSupportsModules (SDKType sdk_type, uint32_t major, uint32_t minor, uint32_t micro);
+    
+    static bool
+    SDKSupportsModules (SDKType desired_type, const lldb_private::FileSpec &sdk_path);
+    
+    struct SDKEnumeratorInfo {
+        lldb_private::FileSpec  found_path;
+        SDKType                 sdk_type;
+    };
+    
+    static lldb_private::FileSpec::EnumerateDirectoryResult
+    DirectoryEnumerator(void *baton,
+                        lldb_private::FileSpec::FileType file_type,
+                        const lldb_private::FileSpec &spec);
+    
+    static lldb_private::FileSpec
+    FindSDKInXcodeForModules (SDKType sdk_type,
+                              const lldb_private::FileSpec &sdks_spec);
+
+    static lldb_private::FileSpec
+    GetSDKDirectoryForModules (PlatformDarwin::SDKType sdk_type);
+
+    void
+    AddClangModuleCompilationOptionsForSDKType (std::vector<std::string> &options, SDKType sdk_type);
 
     std::string                 m_developer_directory;
 
