@@ -31,88 +31,137 @@ public:
     //------------------------------------------------------------
     // lldb_private::Platform functions
     //------------------------------------------------------------
-    virtual lldb_private::OptionGroupOptions*
-    GetConnectionOptions (lldb_private::CommandInterpreter& interpreter);
-    
+    virtual lldb_private::OptionGroupOptions
+    *GetConnectionOptions(
+        lldb_private::CommandInterpreter &interpreter) override;
+
+    const char *
+    GetHostname () override;
+
+    const char *
+    GetUserName (uint32_t uid) override;
+
+    const char *
+    GetGroupName (uint32_t gid) override;
+
     virtual lldb_private::Error
     PutFile (const lldb_private::FileSpec& source,
              const lldb_private::FileSpec& destination,
              uint32_t uid = UINT32_MAX,
-             uint32_t gid = UINT32_MAX);
+             uint32_t gid = UINT32_MAX) override;
     
     virtual lldb::user_id_t
     OpenFile (const lldb_private::FileSpec& file_spec,
               uint32_t flags,
               uint32_t mode,
-              lldb_private::Error &error);
+              lldb_private::Error &error) override;
     
     virtual bool
     CloseFile (lldb::user_id_t fd,
-               lldb_private::Error &error);
+               lldb_private::Error &error) override;
     
     virtual uint64_t
     ReadFile (lldb::user_id_t fd,
               uint64_t offset,
               void *dst,
               uint64_t dst_len,
-              lldb_private::Error &error);
+              lldb_private::Error &error) override;
     
     virtual uint64_t
     WriteFile (lldb::user_id_t fd,
                uint64_t offset,
                const void* src,
                uint64_t src_len,
-               lldb_private::Error &error);
+               lldb_private::Error &error) override;
     
     virtual lldb::user_id_t
-    GetFileSize (const lldb_private::FileSpec& file_spec);
+    GetFileSize (const lldb_private::FileSpec& file_spec) override;
 
     virtual lldb_private::Error
-    CreateSymlink(const char *src, const char *dst);
+    CreateSymlink(const char *src, const char *dst) override;
 
     virtual lldb_private::Error
     GetFile (const lldb_private::FileSpec& source,
-             const lldb_private::FileSpec& destination);
+             const lldb_private::FileSpec& destination) override;
     
     virtual lldb_private::ConstString
-    GetRemoteWorkingDirectory();
+    GetRemoteWorkingDirectory() override;
     
     virtual bool
-    SetRemoteWorkingDirectory(const lldb_private::ConstString &path);
-    
+    SetRemoteWorkingDirectory(const lldb_private::ConstString &path) override;
+
+    bool
+    GetRemoteOSVersion () override;
+
+    bool
+    GetRemoteOSBuildString (std::string &s) override;
+
+    bool
+    GetRemoteOSKernelDescription (std::string &s) override;
+
+    lldb_private::ArchSpec
+    GetRemoteSystemArchitecture () override;
+
+    size_t
+    GetEnvironment (lldb_private::StringList &environment) override;
+
+    bool
+    IsConnected () const override;
+
     virtual lldb_private::Error
     RunShellCommand (const char *command,           // Shouldn't be NULL
                      const char *working_dir,       // Pass NULL to use the current working directory
                      int *status_ptr,               // Pass NULL if you don't want the process exit status
                      int *signo_ptr,                // Pass NULL if you don't want the signal that caused the process to exit
                      std::string *command_output,   // Pass NULL if you don't want the command output
-                     uint32_t timeout_sec);         // Timeout in seconds to wait for shell program to finish
+                     uint32_t timeout_sec) override;// Timeout in seconds to wait for shell program to finish
     
     virtual lldb_private::Error
-    MakeDirectory (const char *path, uint32_t mode);
+    MakeDirectory (const char *path, uint32_t mode) override;
     
     virtual lldb_private::Error
-    GetFilePermissions (const char *path, uint32_t &file_permissions);
+    GetFilePermissions (const char *path, uint32_t &file_permissions) override;
 
     virtual lldb_private::Error
-    SetFilePermissions (const char *path, uint32_t file_permissions);
+    SetFilePermissions (const char *path, uint32_t file_permissions) override;
 
     virtual bool
-    GetFileExists (const lldb_private::FileSpec& file_spec);
+    GetFileExists (const lldb_private::FileSpec& file_spec) override;
     
     virtual lldb_private::Error
-    Unlink (const char *path);
+    Unlink (const char *path) override;
+
+    lldb_private::Error
+    LaunchProcess (lldb_private::ProcessLaunchInfo &launch_info) override;
+
+    lldb::ProcessSP
+    Attach (lldb_private::ProcessAttachInfo &attach_info,
+            lldb_private::Debugger &debugger,
+            lldb_private::Target *target,       // Can be NULL, if NULL create a new target, else use existing one
+            lldb_private::Error &error) override;
+
+    lldb::ProcessSP
+    DebugProcess (lldb_private::ProcessLaunchInfo &launch_info,
+                  lldb_private::Debugger &debugger,
+                  lldb_private::Target *target,       // Can be NULL, if NULL create a new target, else use existing one
+                  lldb_private::Error &error) override;
 
     virtual std::string
-    GetPlatformSpecificConnectionInformation();
+    GetPlatformSpecificConnectionInformation() override;
     
     virtual bool
     CalculateMD5 (const lldb_private::FileSpec& file_spec,
                   uint64_t &low,
-                  uint64_t &high);
+                  uint64_t &high) override;
 
     virtual void
-    CalculateTrapHandlerSymbolNames ();
+    CalculateTrapHandlerSymbolNames () override;
+
+    lldb_private::Error
+    ConnectRemote (lldb_private::Args& args) override;
+
+    lldb_private::Error
+    DisconnectRemote () override;
 
 protected:
     std::unique_ptr<lldb_private::OptionGroupOptions> m_options;
