@@ -18,16 +18,20 @@ class SBBreakpointCallbackCase(TestBase):
           self.buildProgram('inferior.cpp', self.inferior)
           self.addTearDownHook(lambda: os.remove(self.inferior))
 
-    @unittest2.expectedFailure # llvm.org/pr16000: SBBreakpoint.SetCallback() does nothing
+    @unittest2.expectedFailure("llvm.org/pr16000: SBBreakpoint.SetCallback() does nothing")
     @skipIfi386
+    @skipIfRemote
     @skipIfLinuxClang # buildbot clang version unable to use libstdc++ with c++11
+    @skipIfNoSBHeaders
     def test_breakpoint_callback(self):
         """Test the that SBBreakpoint callback is invoked when a breakpoint is hit. """
         self.build_and_test('driver.cpp test_breakpoint_callback.cpp',
                             'test_breakpoint_callback')
 
     @skipIfi386
+    @skipIfRemote
     @skipIfLinuxClang # buildbot clang version unable to use libstdc++ with c++11
+    @skipIfNoSBHeaders
     def test_sb_api_listener_event_description(self):
         """ Test the description of an SBListener breakpoint event is valid."""
         self.build_and_test('driver.cpp listener_test.cpp test_listener_event_description.cpp',
@@ -35,7 +39,9 @@ class SBBreakpointCallbackCase(TestBase):
         pass
 
     @skipIfi386
+    @skipIfRemote
     @skipIfLinuxClang # buildbot clang version unable to use libstdc++ with c++11
+    @skipIfNoSBHeaders
     def test_sb_api_listener_event_process_state(self):
         """ Test that a registered SBListener receives events when a process
             changes state.
@@ -46,7 +52,9 @@ class SBBreakpointCallbackCase(TestBase):
 
 
     @skipIfi386
+    @skipIfRemote
     @skipIfLinuxClang # buildbot clang version unable to use libstdc++ with c++11
+    @skipIfNoSBHeaders
     def test_sb_api_listener_resume(self):
         """ Test that a process can be resumed from a non-main thread. """
         self.build_and_test('driver.cpp listener_test.cpp test_listener_resume.cpp',
