@@ -52,9 +52,11 @@ class BreakpointCommandTestCase(TestBase):
         lldbutil.run_break_set_by_file_and_line (self, None, self.line, num_expected_locations=1, loc_exact=True)
         lldbutil.run_break_set_by_file_and_line (self, "main.c", self.line, num_expected_locations=1, loc_exact=True)
         lldbutil.run_break_set_by_file_and_line (self, "main.c", self.line, num_expected_locations=1, loc_exact=True)
+        # Breakpoint 4 - set at the same location as breakpoint 1 to test setting breakpoint commands on two breakpoints at a time
+        lldbutil.run_break_set_by_file_and_line (self, None, self.line, num_expected_locations=1, loc_exact=True)
 
         # Now add callbacks for the breakpoints just created.
-        self.runCmd("breakpoint command add -s command -o 'frame variable --show-types --scope' 1")
+        self.runCmd("breakpoint command add -s command -o 'frame variable --show-types --scope' 1 4")
         self.runCmd("breakpoint command add -s python -o 'here = open(\"output.txt\", \"w\"); print >> here, \"lldb\"; here.close()' 2")
         self.runCmd("breakpoint command add --python-function bktptcmd.function 3")
 
@@ -83,6 +85,12 @@ class BreakpointCommandTestCase(TestBase):
             substrs = ["Breakpoint commands:",
                           "bktptcmd.function(frame, bp_loc, internal_dict)"])
 
+        self.expect("breakpoint command list 4", "Breakpoint 4 command ok",
+            substrs = ["Breakpoint commands:",
+                          "frame variable --show-types --scope"])
+
+        self.runCmd("breakpoint delete 4")
+
         self.runCmd("command script import --allow-reload ./bktptcmd.py")
 
         # Next lets try some other breakpoint kinds.  First break with a regular expression
