@@ -19,6 +19,7 @@ class SkipSummaryDataFormatterTestCase(TestBase):
         self.buildDsym()
         self.data_formatter_commands()
 
+    @expectedFailureFreeBSD("llvm.org/pr20548") # fails to build on lab.llvm.org buildbot
     @dwarf_test
     def test_with_dwarf_and_run_command(self):
         """Test data formatter commands."""
@@ -135,7 +136,7 @@ class SkipSummaryDataFormatterTestCase(TestBase):
         # Skip the following tests if the condition is met.
         if self.getCompiler().endswith('gcc') and not self.getCompiler().endswith('llvm-gcc'):
            import re
-           gcc_version_output = system([lldbutil.which(self.getCompiler()), "-v"])[1]
+           gcc_version_output = system([[lldbutil.which(self.getCompiler()), "-v"]])[1]
            #print "my output:", gcc_version_output
            for line in gcc_version_output.split(os.linesep):
                m = re.search('\(Apple Inc\. build ([0-9]+)\)', line)
