@@ -67,6 +67,19 @@ class LibcxxStringDataFormatterTestCase(TestBase):
 
         self.runCmd("n")
 
+        TheVeryLongOne = self.frame().FindVariable("TheVeryLongOne");
+        summaryOptions = lldb.SBTypeSummaryOptions()
+        summaryOptions.SetCapping(lldb.eTypeSummaryUncapped)
+        uncappedSummaryStream = lldb.SBStream()
+        TheVeryLongOne.GetSummary(uncappedSummaryStream,summaryOptions)
+        uncappedSummary = uncappedSummaryStream.GetData()
+        self.assertTrue(uncappedSummary.find("someText") > 0, "uncappedSummary does not include the full string")
+        summaryOptions.SetCapping(lldb.eTypeSummaryCapped)
+        cappedSummaryStream = lldb.SBStream()
+        TheVeryLongOne.GetSummary(cappedSummaryStream,summaryOptions)
+        cappedSummary = cappedSummaryStream.GetData()
+        self.assertTrue(cappedSummary.find("someText") <= 0, "cappedSummary includes the full string")
+
         self.expect("frame variable",
                     substrs = ['(std::__1::wstring) s = L"hello world! מזל טוב!"',
                     '(std::__1::wstring) S = L"!!!!!"',
