@@ -8,6 +8,7 @@ import os.path
 import platform
 import random
 import re
+import select
 import sets
 import signal
 import socket
@@ -31,7 +32,11 @@ class GdbRemoteTestCaseBase(TestBase):
     _LOGGING_LEVEL = logging.WARNING
     # _LOGGING_LEVEL = logging.DEBUG
 
+    # Start the inferior separately, attach to the inferior on the stub command line.
     _STARTUP_ATTACH = "attach"
+    # Start the inferior separately, start the stub without attaching, allow the test to attach to the inferior however it wants (e.g. $vAttach;pid).
+    _STARTUP_ATTACH_MANUALLY = "attach_manually"
+    # Start the stub, and launch the inferior with an $A packet via the initial packet stream.
     _STARTUP_LAUNCH = "launch"
 
     # GDB Signal numbers that are not target-specific used for common exceptions
@@ -55,6 +60,7 @@ class GdbRemoteTestCaseBase(TestBase):
         self.named_pipe = None
         self.named_pipe_fd = None
         self.stub_sends_two_stop_notifications_on_kill = False
+        self.stub_hostname = "localhost"
 
     def get_next_port(self):
         return 12000 + random.randint(0,3999)
@@ -165,7 +171,7 @@ class GdbRemoteTestCaseBase(TestBase):
 
         self.addTearDownHook(shutdown_socket)
 
-        connect_info = ("localhost", self.port)
+        connect_info = (self.stub_hostname, self.port)
         # print "connecting to stub on {}:{}".format(connect_info[0], connect_info[1])
         sock.connect(connect_info)
 
@@ -177,17 +183,24 @@ class GdbRemoteTestCaseBase(TestBase):
     def set_inferior_startup_attach(self):
         self._inferior_startup = self._STARTUP_ATTACH
 
-    def launch_debug_monitor(self, attach_pid=None):
-        # Create the command line.
-        import pexpect
+    def set_inferior_startup_attach_manually(self):
+        self._inferior_startup = self._STARTUP_ATTACH_MANUALLY
+
+    def get_debug_monitor_command_line(self, attach_pid=None):
         commandline = "{}{} localhost:{}".format(self.debug_monitor_exe, self.debug_monitor_extra_args, self.port)
         if attach_pid:
             commandline += " --attach=%d" % attach_pid
         if self.named_pipe_path:
             commandline += " --named-pipe %s" % self.named_pipe_path
+        return commandline
+
+    def launch_debug_monitor(self, attach_pid=None, logfile=None):
+        # Create the command line.
+        import pexpect
+        commandline = self.get_debug_monitor_command_line(attach_pid=attach_pid)
 
         # Start the server.
-        server = pexpect.spawn(commandline)
+        server = pexpect.spawn(commandline, logfile=logfile)
         self.assertIsNotNone(server)
         server.expect(r"(debugserver|lldb-gdbserver)", timeout=10)
 
@@ -271,11 +284,12 @@ class GdbRemoteTestCaseBase(TestBase):
 
         raise Exception("failed to create a socket to the launched debug monitor after %d tries" % attempts)
 
-    def launch_process_for_attach(self,inferior_args=None, sleep_seconds=3):
+    def launch_process_for_attach(self,inferior_args=None, sleep_seconds=3, exe_path=None):
         # We're going to start a child process that the debug monitor stub can later attach to.
         # This process needs to be started so that it just hangs around for a while.  We'll
         # have it sleep.
-        exe_path = os.path.abspath("a.out")
+        if not exe_path:
+            exe_path = os.path.abspath("a.out")
 
         args = [exe_path]
         if inferior_args:
@@ -285,7 +299,7 @@ class GdbRemoteTestCaseBase(TestBase):
 
         return subprocess.Popen(args)
 
-    def prep_debug_monitor_and_inferior(self, inferior_args=None, inferior_sleep_seconds=3):
+    def prep_debug_monitor_and_inferior(self, inferior_args=None, inferior_sleep_seconds=3, inferior_exe_path=None):
         """Prep the debug monitor, the inferior, and the expected packet stream.
 
         Handle the separate cases of using the debug monitor in attach-to-inferior mode
@@ -308,12 +322,14 @@ class GdbRemoteTestCaseBase(TestBase):
         inferior = None
         attach_pid = None
 
-        if self._inferior_startup == self._STARTUP_ATTACH:
+        if self._inferior_startup == self._STARTUP_ATTACH or self._inferior_startup == self._STARTUP_ATTACH_MANUALLY:
             # Launch the process that we'll use as the inferior.
-            inferior = self.launch_process_for_attach(inferior_args=inferior_args, sleep_seconds=inferior_sleep_seconds)
+            inferior = self.launch_process_for_attach(inferior_args=inferior_args, sleep_seconds=inferior_sleep_seconds, exe_path=inferior_exe_path)
             self.assertIsNotNone(inferior)
             self.assertTrue(inferior.pid > 0)
-            attach_pid = inferior.pid
+            if self._inferior_startup == self._STARTUP_ATTACH:
+                # In this case, we want the stub to attach via the command line, so set the command line attach pid here.
+                attach_pid = inferior.pid
 
         # Launch the debug monitor stub, attaching to the inferior.
         server = self.connect_to_debug_monitor(attach_pid=attach_pid)
@@ -321,7 +337,9 @@ class GdbRemoteTestCaseBase(TestBase):
 
         if self._inferior_startup == self._STARTUP_LAUNCH:
             # Build launch args
-            launch_args = [os.path.abspath('a.out')]
+            if not inferior_exe_path:
+                inferior_exe_path = os.path.abspath("a.out")
+            launch_args = [inferior_exe_path]
             if inferior_args:
                 launch_args.extend(inferior_args)
 
@@ -332,6 +350,45 @@ class GdbRemoteTestCaseBase(TestBase):
 
         return {"inferior":inferior, "server":server}
 
+    def expect_socket_recv(self, sock, expected_content_regex, timeout_seconds):
+        response = ""
+        timeout_time = time.time() + timeout_seconds
+
+        while not expected_content_regex.match(response) and time.time() < timeout_time: 
+            can_read, _, _ = select.select([sock], [], [], timeout_seconds)
+            if can_read and sock in can_read:
+                recv_bytes = sock.recv(4096)
+                if recv_bytes:
+                    response += recv_bytes
+
+        self.assertTrue(expected_content_regex.match(response))
+
+    def expect_socket_send(self, sock, content, timeout_seconds):
+        request_bytes_remaining = content
+        timeout_time = time.time() + timeout_seconds
+
+        while len(request_bytes_remaining) > 0 and time.time() < timeout_time:
+            _, can_write, _ = select.select([], [sock], [], timeout_seconds)
+            if can_write and sock in can_write:
+                written_byte_count = sock.send(request_bytes_remaining)
+                request_bytes_remaining = request_bytes_remaining[written_byte_count:]
+        self.assertEquals(len(request_bytes_remaining), 0)
+
+    def do_handshake(self, stub_socket, timeout_seconds=5):
+        # Write the ack.
+        self.expect_socket_send(stub_socket, "+", timeout_seconds)
+
+        # Send the start no ack mode packet.
+        NO_ACK_MODE_REQUEST = "$QStartNoAckMode#b0"
+        bytes_sent = stub_socket.send(NO_ACK_MODE_REQUEST)
+        self.assertEquals(bytes_sent, len(NO_ACK_MODE_REQUEST))
+
+        # Receive the ack and "OK"
+        self.expect_socket_recv(stub_socket, re.compile(r"^\+\$OK#[0-9a-fA-F]{2}$"), timeout_seconds)
+
+        # Send the final ack.
+        self.expect_socket_send(stub_socket, "+", timeout_seconds)
+
     def add_no_ack_remote_stream(self):
         self.test_sequence.add_log_lines(
             ["read packet: +",
@@ -351,13 +408,13 @@ class GdbRemoteTestCaseBase(TestBase):
 
     def add_thread_suffix_request_packets(self):
         self.test_sequence.add_log_lines(
-            ["read packet: $QThreadSuffixSupported#00",
+            ["read packet: $QThreadSuffixSupported#e4",
              "send packet: $OK#00",
             ], True)
 
     def add_process_info_collection_packets(self):
         self.test_sequence.add_log_lines(
-            ["read packet: $qProcessInfo#00",
+            ["read packet: $qProcessInfo#dc",
               { "direction":"send", "regex":r"^\$(.+)#[0-9a-fA-F]{2}$", "capture":{1:"process_info_raw"} }],
             True)
 
@@ -371,6 +428,7 @@ class GdbRemoteTestCaseBase(TestBase):
         "cputype",
         "cpusubtype",
         "ostype",
+        "triple",
         "vendor",
         "endian",
         "ptrsize"
@@ -568,7 +626,7 @@ class GdbRemoteTestCaseBase(TestBase):
         if (do_continue):
             self.test_sequence.add_log_lines(
                 [# Continue the inferior.
-                 "read packet: $c#00",
+                 "read packet: $c#63",
                  # Expect a breakpoint stop report.
                  {"direction":"send", "regex":r"^\$T([0-9a-fA-F]{2})thread:([0-9a-fA-F]+);", "capture":{1:"stop_signo", 2:"stop_thread_id"} },
                  ], True)        
@@ -631,7 +689,7 @@ class GdbRemoteTestCaseBase(TestBase):
     def run_process_then_stop(self, run_seconds=1):
         # Tell the stub to continue.
         self.test_sequence.add_log_lines(
-             ["read packet: $vCont;c#00"],
+             ["read packet: $vCont;c#a8"],
              True)
         context = self.expect_gdbremote_sequence()
 
@@ -676,7 +734,7 @@ class GdbRemoteTestCaseBase(TestBase):
 
         registers = {}
         for (key, val) in kv_dict.items():
-            if re.match(r"^[0-9a-fA-F]+", key):
+            if re.match(r"^[0-9a-fA-F]+$", key):
                 registers[int(key, 16)] = val
         return registers
 
@@ -977,7 +1035,7 @@ class GdbRemoteTestCaseBase(TestBase):
 
     def add_vCont_query_packets(self):
         self.test_sequence.add_log_lines([
-            "read packet: $vCont?#00",
+            "read packet: $vCont?#49",
             {"direction":"send", "regex":r"^\$(vCont)?(.*)#[0-9a-fA-F]{2}$", "capture":{2:"vCont_query_response" } },
             ], True)
 
@@ -1066,7 +1124,7 @@ class GdbRemoteTestCaseBase(TestBase):
         # Run the process
         self.test_sequence.add_log_lines(
             [# Start running after initial stop.
-             "read packet: $c#00",
+             "read packet: $c#63",
              # Match output line that prints the memory address of the function call entry point.
              # Note we require launch-only testing so we can get inferior otuput.
              { "type":"output_match", "regex":r"^code address: 0x([0-9a-fA-F]+)\r\ndata address: 0x([0-9a-fA-F]+)\r\ndata address: 0x([0-9a-fA-F]+)\r\n$", 
