@@ -7,8 +7,6 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include "lldb/lldb-python.h"
-
 // C Includes
 #include <errno.h>
 #include <getopt.h>
@@ -27,12 +25,16 @@
 // Other libraries and framework includes
 #include "lldb/lldb-private-log.h"
 #include "lldb/Core/Error.h"
-#include "lldb/Core/ConnectionFileDescriptor.h"
 #include "lldb/Core/ConnectionMachPort.h"
 #include "lldb/Core/Debugger.h"
 #include "lldb/Core/PluginManager.h"
 #include "lldb/Core/StreamFile.h"
+#include "lldb/Host/ConnectionFileDescriptor.h"
+#include "lldb/Host/HostThread.h"
+#include "lldb/Host/Pipe.h"
 #include "lldb/Host/OptionParser.h"
+#include "lldb/Host/Socket.h"
+#include "lldb/Host/ThreadLauncher.h"
 #include "lldb/Interpreter/CommandInterpreter.h"
 #include "lldb/Interpreter/CommandReturnObject.h"
 #include "Plugins/Process/gdb-remote/GDBRemoteCommunicationServer.h"
@@ -53,9 +55,9 @@ using namespace lldb_private;
 
 namespace
 {
-    static lldb::thread_t s_listen_thread = LLDB_INVALID_HOST_THREAD;
-    static std::unique_ptr<ConnectionFileDescriptor> s_listen_connection_up;
-    static std::string s_listen_url;
+HostThread s_listen_thread;
+    std::unique_ptr<ConnectionFileDescriptor> s_listen_connection_up;
+    std::string s_listen_url;
 }
 
 //----------------------------------------------------------------------
@@ -76,6 +78,7 @@ static struct option g_long_options[] =
     { "attach",             required_argument,  NULL,               'a' },
     { "named-pipe",         required_argument,  NULL,               'P' },
     { "native-regs",        no_argument,        NULL,               'r' },  // Specify to use the native registers instead of the gdb defaults for the architecture.  NOTE: this is a do-nothing arg as it's behavior is default now.  FIXME remove call from lldb-platform.
+    { "reverse-connect",    no_argument,        NULL,               'R' },  // Specifies that llgs attaches to the client address:port rather than llgs listening for a connection from address on port.
     { "setsid",             no_argument,        NULL,               'S' },  // Call setsid() to make llgs run in its own session.
     { NULL,                 0,                  NULL,               0   }
 };
@@ -138,31 +141,17 @@ dump_available_platforms (FILE *output_file)
         fprintf (output_file, "%s\t%s\n", plugin_name, plugin_desc);
     }
 
-    if ( Platform::GetDefaultPlatform () )
+    if ( Platform::GetHostPlatform () )
     {
         // add this since the default platform doesn't necessarily get registered by
         // the plugin name (e.g. 'host' doesn't show up as a
         // registered platform plugin even though it's the default).
-        fprintf (output_file, "%s\tDefault platform for this host.\n", Platform::GetDefaultPlatform ()->GetPluginName ().AsCString ());
+        fprintf (output_file, "%s\tDefault platform for this host.\n", Platform::GetHostPlatform ()->GetPluginName ().AsCString ());
     }
 }
 
 static void
-initialize_lldb_gdbserver ()
-{
-    PluginManager::Initialize ();
-    Debugger::Initialize (NULL);
-}
-
-static void
-terminate_lldb_gdbserver ()
-{
-    Debugger::Terminate();
-    PluginManager::Terminate ();
-}
-
-static void
-run_lldb_commands (const lldb::DebuggerSP &debugger_sp, const std::vector<std::string> lldb_commands)
+run_lldb_commands (const lldb::DebuggerSP &debugger_sp, const std::vector<std::string> &lldb_commands)
 {
     for (const auto &lldb_command : lldb_commands)
     {
@@ -177,28 +166,28 @@ run_lldb_commands (const lldb::DebuggerSP &debugger_sp, const std::vector<std::s
 }
 
 static lldb::PlatformSP
-setup_platform (const std::string platform_name)
+setup_platform (const std::string &platform_name)
 {
     lldb::PlatformSP platform_sp;
 
     if (platform_name.empty())
     {
         printf ("using the default platform: ");
-        platform_sp = Platform::GetDefaultPlatform ();
+        platform_sp = Platform::GetHostPlatform ();
         printf ("%s\n", platform_sp->GetPluginName ().AsCString ());
         return platform_sp;
     }
 
     Error error;
-    platform_sp = Platform::Create (platform_name.c_str(), error);
+    platform_sp = Platform::Create (lldb_private::ConstString(platform_name), error);
     if (error.Fail ())
     {
         // the host platform isn't registered with that name (at
         // least, not always.  Check if the given name matches
         // the default platform name.  If so, use it.
-        if ( Platform::GetDefaultPlatform () && ( Platform::GetDefaultPlatform ()->GetPluginName () == ConstString (platform_name.c_str()) ) )
+        if ( Platform::GetHostPlatform () && ( Platform::GetHostPlatform ()->GetPluginName () == ConstString (platform_name.c_str()) ) )
         {
-            platform_sp = Platform::GetDefaultPlatform ();
+            platform_sp = Platform::GetHostPlatform ();
         }
         else
         {
@@ -291,7 +280,7 @@ static Error
 StartListenThread (const char *hostname, uint16_t port)
 {
     Error error;
-    if (IS_VALID_LLDB_HOST_THREAD(s_listen_thread))
+    if (s_listen_thread.IsJoinable())
     {
         error.SetErrorString("listen thread already running");
     }
@@ -305,7 +294,7 @@ StartListenThread (const char *hostname, uint16_t port)
 
         s_listen_url = listen_url;
         s_listen_connection_up.reset (new ConnectionFileDescriptor ());
-        s_listen_thread = Host::ThreadCreate (listen_url, ListenThread, nullptr, &error);
+        s_listen_thread = ThreadLauncher::LaunchThread(listen_url, ListenThread, nullptr, &error);
     }
     return error;
 }
@@ -313,25 +302,40 @@ StartListenThread (const char *hostname, uint16_t port)
 static bool
 JoinListenThread ()
 {
-    if (IS_VALID_LLDB_HOST_THREAD(s_listen_thread))
-    {
-        Host::ThreadJoin(s_listen_thread, nullptr, nullptr);
-        s_listen_thread = LLDB_INVALID_HOST_THREAD;
-    }
+    if (s_listen_thread.IsJoinable())
+        s_listen_thread.Join(nullptr);
     return true;
 }
 
+Error
+writePortToPipe (const char *const named_pipe_path, const uint16_t port)
+{
+    Pipe port_name_pipe;
+    // Wait for 10 seconds for pipe to be opened.
+    auto error = port_name_pipe.OpenAsWriterWithTimeout (named_pipe_path, false, std::chrono::microseconds (10 * 1000000));
+    if (error.Fail ())
+        return error;
+
+    char port_str[64];
+    const auto port_str_len = ::snprintf (port_str, sizeof (port_str), "%u", port);
+
+    size_t bytes_written = 0;
+    // Write the port number as a C string with the NULL terminator.
+    return port_name_pipe.Write (port_str, port_str_len + 1, bytes_written);
+}
+
 void
-start_listener (GDBRemoteCommunicationServer &gdb_server, const char *const host_and_port, const char *const progname, const char *const named_pipe_path)
+ConnectToRemote (GDBRemoteCommunicationServer &gdb_server, bool reverse_connect, const char *const host_and_port, const char *const progname, const char *const named_pipe_path)
 {
     Error error;
 
     if (host_and_port && host_and_port[0])
     {
+        // Parse out host and port.
         std::string final_host_and_port;
-        std::string listening_host;
-        std::string listening_port;
-        uint32_t listening_portno = 0;
+        std::string connection_host;
+        std::string connection_port;
+        uint32_t connection_portno = 0;
 
         // If host_and_port starts with ':', default the host to be "localhost" and expect the remainder to be the port.
         if (host_and_port[0] == ':')
@@ -341,9 +345,9 @@ start_listener (GDBRemoteCommunicationServer &gdb_server, const char *const host
         const std::string::size_type colon_pos = final_host_and_port.find (':');
         if (colon_pos != std::string::npos)
         {
-            listening_host = final_host_and_port.substr (0, colon_pos);
-            listening_port = final_host_and_port.substr (colon_pos + 1);
-            listening_portno = Args::StringToUInt32 (listening_port.c_str (), 0);
+            connection_host = final_host_and_port.substr (0, colon_pos);
+            connection_port = final_host_and_port.substr (colon_pos + 1);
+            connection_portno = Args::StringToUInt32 (connection_port.c_str (), 0);
         }
         else
         {
@@ -352,51 +356,86 @@ start_listener (GDBRemoteCommunicationServer &gdb_server, const char *const host
             exit (1);
         }
 
-        // Start the listener on a new thread.  We need to do this so we can resolve the
-        // bound listener port.
-        StartListenThread(listening_host.c_str (), static_cast<uint16_t> (listening_portno));
-        printf ("Listening to port %s for a connection from %s...\n", listening_port.c_str (), listening_host.c_str ());
-
-        // If we have a named pipe to write the port number back to, do that now.
-        if (named_pipe_path && named_pipe_path[0] && listening_portno == 0)
+        if (reverse_connect)
         {
-            // FIXME use new generic named pipe support.
-            int fd = ::open(named_pipe_path, O_WRONLY);
-            if (fd > -1)
+            // llgs will connect to the gdb-remote client.
+
+            // Ensure we have a port number for the connection.
+            if (connection_portno == 0)
             {
-                const uint16_t bound_port = s_listen_connection_up->GetBoundPort (10);
-
-                char port_str[64];
-                const ssize_t port_str_len = ::snprintf (port_str, sizeof(port_str), "%u", bound_port);
-                // Write the port number as a C string with the NULL terminator.
-                ::write (fd, port_str, port_str_len + 1);
-                close (fd);
+                fprintf (stderr, "error: port number must be specified on when using reverse connect");
+                exit (1);
             }
-            else
+
+            // Build the connection string.
+            char connection_url[512];
+            snprintf(connection_url, sizeof(connection_url), "connect://%s", final_host_and_port.c_str ());
+
+            // Create the connection.
+            std::unique_ptr<ConnectionFileDescriptor> connection_up (new ConnectionFileDescriptor ());
+            connection_up.reset (new ConnectionFileDescriptor ());
+            auto connection_result = connection_up->Connect (connection_url, &error);
+            if (connection_result != eConnectionStatusSuccess)
             {
-                fprintf (stderr, "failed to open named pipe '%s' for writing\n", named_pipe_path);
+                fprintf (stderr, "error: failed to connect to client at '%s' (connection status: %d)", connection_url, static_cast<int> (connection_result));
+                exit (-1);
+            }
+            if (error.Fail ())
+            {
+                fprintf (stderr, "error: failed to connect to client at '%s': %s", connection_url, error.AsCString ());
+                exit (-1);
             }
-        }
 
-        // Join the listener thread.
-        if (!JoinListenThread ())
-        {
-            fprintf (stderr, "failed to join the listener thread\n");
-            display_usage (progname);
-            exit (1);
-        }
-
-        // Ensure we connected.
-        if (s_listen_connection_up)
-        {
+            // We're connected.
             printf ("Connection established.\n");
-            gdb_server.SetConnection (s_listen_connection_up.release());
+            gdb_server.SetConnection (connection_up.release());
         }
         else
         {
-            fprintf (stderr, "failed to connect to '%s': %s\n", final_host_and_port.c_str (), error.AsCString ());
-            display_usage (progname);
-            exit (1);
+            // llgs will listen for connections on the given port from the given address.
+            // Start the listener on a new thread.  We need to do this so we can resolve the
+            // bound listener port.
+            StartListenThread(connection_host.c_str (), static_cast<uint16_t> (connection_portno));
+            printf ("Listening to port %s for a connection from %s...\n", connection_port.c_str (), connection_host.c_str ());
+
+            // If we have a named pipe to write the port number back to, do that now.
+            if (named_pipe_path && named_pipe_path[0] && connection_portno == 0)
+            {
+                const uint16_t bound_port = s_listen_connection_up->GetListeningPort (10);
+                if (bound_port > 0)
+                {
+                    error = writePortToPipe (named_pipe_path, bound_port);
+                    if (error.Fail ())
+                    {
+                        fprintf (stderr, "failed to write to the named pipe \'%s\': %s", named_pipe_path, error.AsCString());
+                    }
+                }
+                else
+                {
+                    fprintf (stderr, "unable to get the bound port for the listening connection\n");
+                }
+            }
+
+            // Join the listener thread.
+            if (!JoinListenThread ())
+            {
+                fprintf (stderr, "failed to join the listener thread\n");
+                display_usage (progname);
+                exit (1);
+            }
+
+            // Ensure we connected.
+            if (s_listen_connection_up)
+            {
+                printf ("Connection established.\n");
+                gdb_server.SetConnection (s_listen_connection_up.release());
+            }
+            else
+            {
+                fprintf (stderr, "failed to connect to '%s': %s\n", final_host_and_port.c_str (), error.AsCString ());
+                display_usage (progname);
+                exit (1);
+            }
         }
     }
 
@@ -411,7 +450,7 @@ start_listener (GDBRemoteCommunicationServer &gdb_server, const char *const host
 
             bool interrupt = false;
             bool done = false;
-            while (!interrupt && !done && (g_sighup_received_count < 1))
+            while (!interrupt && !done && (g_sighup_received_count < 2))
             {
                 const GDBRemoteCommunication::PacketResult result = gdb_server.GetPacketAndSendResponse (TIMEOUT_USEC, error, interrupt, done);
                 if ((result != GDBRemoteCommunication::PacketResult::Success) &&
@@ -462,8 +501,9 @@ main (int argc, char *argv[])
     std::string platform_name;
     std::string attach_target;
     std::string named_pipe_path;
+    bool reverse_connect = false;
 
-    initialize_lldb_gdbserver ();
+    Debugger::Initialize (NULL);
 
     lldb::DebuggerSP debugger_sp = Debugger::CreateInstance ();
 
@@ -547,6 +587,10 @@ main (int argc, char *argv[])
             // Do nothing, native regs is the default these days
             break;
 
+        case 'R':
+            reverse_connect = true;
+            break;
+
 #ifndef _WIN32
         case 'S':
             // Put llgs into a new session. Terminals group processes
@@ -631,9 +675,9 @@ main (int argc, char *argv[])
     // Print version info.
     printf("%s-%s", LLGS_PROGRAM_NAME, LLGS_VERSION_STR);
 
-    start_listener (gdb_server, host_and_port, progname, named_pipe_path.c_str ());
+    ConnectToRemote (gdb_server, reverse_connect, host_and_port, progname, named_pipe_path.c_str ());
 
-    terminate_lldb_gdbserver ();
+    Debugger::Terminate ();
 
     fprintf(stderr, "lldb-gdbserver exiting...\n");
 
